import json
import getpass
from pathlib import Path
from proton.api import Session
from proton.exceptions import NetworkError, ProtonAPIError


class ProtonMail:
    def __init__(self, cache_dir: Path, session_path: Path, proxies: list[[str, str]] | None):
        self.cache_dir = cache_dir
        self.session_path = session_path
        self._session = None
        if proxies:
            self.proxies = dict(proxies)
        else:
            self.proxies = None

    @property
    def session(self):
        if self._session is None:
            with open(self.session_path, 'r') as f:
                session_dump = json.loads(f.read())

            self._session = Session.load(
                dump=session_dump,
                log_dir_path=self.cache_dir,
                cache_dir_path=self.cache_dir,
                tls_pinning=not self.proxies,
                proxies=self.proxies
            )
            self._session.enable_alternative_routing = True
        return self._session

    def authenticate(self):
        session = Session(
            api_url='https://api.protonmail.ch',
            log_dir_path=self.cache_dir,
            cache_dir_path=self.cache_dir,
            tls_pinning=not self.proxies,
            proxies=self.proxies
        )
        session.enable_alternative_routing = True

        username = input('Username: ')
        password = getpass.getpass()
        scope = session.authenticate(username, password)

        if 'twofactor' in scope:
            code = input('Two-factor auth: ')
            session.provide_2fa(code)

        with open(self.session_path, 'w') as f:
            json.dump(session.dump(), f)

    def get_unread(self):
        resp = self.session.api_request('/messages/count')
        inbox = next(c for c in resp['Counts'] if c['LabelID'] == '0')
        return inbox['Unread']

    def get_latest_event(self):
        return self.session.api_request('/events/latest')

    def get_events(self, event_id: str):
        return self.session.api_request('/events/' + event_id)

    def refresh_tokens(self):
        try:
            self.session.refresh()
            with open(self.session_path, 'w') as f:
                json.dump(self.session.dump(), f)
            return (True, None)
        except (NetworkError, ProtonAPIError) as error:
            return (False, error)



import sys
import argparse
from pathlib import Path
from bar_protonmail.app import Application, UrgencyLevel
from bar_protonmail.printer import WaybarPrinter, PolybarPrinter
from bar_protonmail.protonmail import ProtonMail


def cli():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='subcommand')
    subparsers.add_parser('auth', help='Authentication.')
    parser.add_argument('-f', '--format', choices=['waybar', 'polybar'], default='waybar',
                        help='Print output in specified format [default: waybar].')
    parser.add_argument('-b', '--badge', default='',
                        help='Badge to display in the bar [default: ].')
    parser.add_argument('-c', '--color',
                        help='Text foreground color (only for Polybar).')
    parser.add_argument('-s', '--sound',
                        help='Notification sound (event sound ID from canberra-gtk-play).')
    parser.add_argument('-u', '--urgency', choices=['low', 'normal', 'critical'],
                        default='normal', help='Notification urgency level [default: normal].')
    parser.add_argument('-t', '--expire-timeout', type=int, default=0,
                        help='The duration, in milliseconds, for the notification to appear on screen.')
    parser.add_argument('-dn', '--no-notify', action='store_true',
                        help='Disable new email notifications.')
    parser.add_argument('-p', '--proxy', action='append', nargs=2, metavar=('PROTOCOL', 'URL'),
                        help='Protocol and URL of the proxy (e.g. "https socks5://user:password@host:port").')
    args = parser.parse_args()

    if args.color is not None and args.format != 'polybar':
        parser.error(
            '`--color COLOR` can be used only with `--format polybar`.')

    cache_dir = Path(Path.home(), '.cache/bar-protonmail')
    session_path = Path(cache_dir, 'session.json')
    unread_path = Path(cache_dir, 'unread.json')

    if not cache_dir.is_dir():
        cache_dir.mkdir(exist_ok=True)

    protonmail = ProtonMail(cache_dir, session_path, proxies=args.proxy)

    if args.subcommand == 'auth':
        protonmail.authenticate()
        print('Session saved successfully.')
        sys.exit()

    if args.format == 'waybar':
        printer = WaybarPrinter(badge=args.badge)
    elif args.format == 'polybar':
        printer = PolybarPrinter(badge=args.badge, color=args.color)

    if not session_path.is_file():
        printer.error('Authentication required')
        sys.exit()

    app = Application(unread_path, protonmail, printer,
                      sound_id=args.sound,
                      urgency_level=UrgencyLevel[args.urgency.upper()],
                      expire_timeout=args.expire_timeout,
                      is_notify=not args.no_notify)
    app.run()


if __name__ == '__main__':
    cli()



import json


class Printer:
    def _print(self, text):
        try:
            print(text, flush=True)
        except BrokenPipeError:
            pass


class WaybarPrinter(Printer):
    def __init__(self, badge: str):
        self.badge = badge

    def print(self, count, inaccurate=False):
        classes = set()
        text = ''
        if inaccurate:
            classes.add('inaccurate')
        if count > 0:
            classes.add('unread')
            text = str(count)
        text = f'{self.badge} {text}'.strip()
        text = json.dumps({'text': f'{text}', 'class': list(classes)})
        self._print(text)

    def error(self, message):
        text = json.dumps({'text': message, 'class': 'error'})
        self._print(text)


class PolybarPrinter(Printer):
    def __init__(self, badge: str, color: str = None):
        self.badge = badge
        self.color = color

    def print(self, count, inaccurate=False):
        text = str(count) if count > 0 else ''
        if inaccurate:
            text = f'~{text}'
        text = f'{self.badge} {text}'.strip()
        if count > 0 and self.color:
            text = f'%{{F{self.color}}}{text}%{{F-}}'
        self._print(text)

    def error(self, message):
        text = f' {message}'
        if self.color:
            text = f'%{{F{self.color}}}{text}%{{F-}}'
        self._print(text)



import os
import json
from enum import Enum
from pathlib import Path
from subprocess import Popen
from gi.repository import GLib
from dasbus.connection import SessionMessageBus
from dasbus.error import DBusError
from proton.exceptions import NetworkError, ProtonAPIError
from bar_protonmail.printer import WaybarPrinter, PolybarPrinter
from bar_protonmail.protonmail import ProtonMail

APP_NAME = 'Bar ProtonMail'
NOTIFICATION_CATEGORY = 'email.arrived'
BASE_DIR = Path(__file__).resolve().parent
PROTONMAIL_ICON_PATH = Path(BASE_DIR, 'icon.svg')


class UrgencyLevel(Enum):
    LOW = 0
    NORMAL = 1
    CRITICAL = 2


class Application:
    def __init__(self, unread_path: Path, protonmail: ProtonMail,
                 printer: WaybarPrinter | PolybarPrinter, sound_id: str,
                 urgency_level: UrgencyLevel, expire_timeout: int, is_notify: bool):
        self.unread_path = unread_path
        self.protonmail = protonmail
        self.printer = printer
        self.sound_id = sound_id
        self.urgency_level = urgency_level
        self.expire_timeout = expire_timeout
        self.is_notify = is_notify

    @staticmethod
    def _message_to_notification(message):
        m = message['Message']
        return {
            'title': m.get('SenderName') or m.get('SenderAddress'),
            'body': m['Subject']
        }

    def _play_sound(self):
        try:
            Popen(['canberra-gtk-play', '-i', self.sound_id], stderr=open(os.devnull, 'wb'))
        except FileNotFoundError:
            pass

    def _send_notifications(self, messages):
        try:
            bus = SessionMessageBus()
            proxy = bus.get_proxy(
                'org.freedesktop.Notifications',
                '/org/freedesktop/Notifications'
            )

            app_icon = str(PROTONMAIL_ICON_PATH)
            replaces_id = 0
            actions = []

            # https://lazka.github.io/pgi-docs/GLib-2.0/classes/VariantType.html#GLib.VariantType
            hints = {
                'category': GLib.Variant('s', NOTIFICATION_CATEGORY),
                'urgency': GLib.Variant('y', self.urgency_level.value),
            }

            for message in messages:
                summary = message['title']
                body = message['body']

                # https://specifications.freedesktop.org/notification-spec/notification-spec-latest.html
                proxy.Notify(APP_NAME, replaces_id, app_icon, summary,
                             body, actions, hints, self.expire_timeout)
        except DBusError:
            pass

    def run(self):
        try:
            if self.unread_path.is_file():
                with open(self.unread_path, 'r') as f:
                    data = json.load(f)
                    unread_prev = data['Unread']
                    event_id = data['EventID']
            else:
                unread_prev = None
                latest = self.protonmail.get_latest_event()
                event_id = latest['EventID']

            events = self.protonmail.get_events(event_id)
            event_id = events['EventID']
            messages = events.get('Messages', [])

            if messages:
                new_messages = filter(
                    lambda m: m['Action'] == 1 and m['Message']['Unread'], messages)
                new_messages = list(map(self._message_to_notification, new_messages))
                if len(new_messages) > 0:
                    if self.sound_id:
                        self._play_sound()
                    if self.is_notify:
                        self._send_notifications(new_messages)

            unread = self.protonmail.get_unread()
            self.printer.print(unread)

            if unread_prev != unread or messages:
                with open(self.unread_path, 'w') as f:
                    json.dump({'EventID': event_id, 'Unread': unread}, f)

        except NetworkError:
            self.printer.print(unread_prev or 0, inaccurate=True)

        except ProtonAPIError as e:
            # Code 401: Invalid access token.
            if e.code == 401:
                is_success, error = self.protonmail.refresh_tokens()
                if is_success:
                    self.run()
                else:
                    self.printer.error(str(error))
            else:
                self.printer.error(str(e))
